#include <windows.h>
#include <commctrl.h>
#include <strsafe.h>
#include "resource.h"
#include "plugin.h"

static HINSTANCE hInstance = NULL;
static MINIGRAPHIC *gfx = NULL;
static POINT position = {0, 0};

extern MINIGRAPHIC *ImageIt(MINIGRAPHICPARAM *param, int preview,
			    int fliph, int flipv, int rot, MINIGRAPHIC *src);


BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd );
#ifdef __cplusplus
extern "C"{
#endif
BOOL WINAPI _DllMainCRTStartup( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  return DllMain(hModule, reason, resvd);
}
#ifdef __cplusplus
};
#endif


BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  hInstance = (HINSTANCE)hModule;
	
  return TRUE;
}

static void CenterDialog( HWND hwndDlg )
{
  HWND hwndOwner; 
  RECT rc, rcDlg, rcOwner; 

  hwndOwner = GetParent(hwndDlg);
  if (hwndOwner == NULL) hwndOwner = GetDesktopWindow(); 
  GetWindowRect(hwndOwner, &rcOwner); 
  GetWindowRect(hwndDlg, &rcDlg); 
  CopyRect(&rc, &rcOwner); 
  OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
  OffsetRect(&rc, -rc.left, -rc.top); 
  OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 
  SetWindowPos(hwndDlg, HWND_TOP, rcOwner.left + (rc.right / 2),
	       rcOwner.top + (rc.bottom / 2), 0, 0, SWP_NOSIZE); 
}

static void PositionWindow( HWND hdlg, POINT *pt)
{
  HDC hdc;
  if (pt->x!=0 && pt->y!=0) {
    hdc=CreateIC("DISPLAY", "", "", NULL);
    pt->x = min( GetDeviceCaps(hdc, HORZRES)-100, pt->x);
    pt->x = max( pt->x, 0 );
    pt->y = min( GetDeviceCaps(hdc, VERTRES)-100, pt->y);
    pt->y = max( pt->y, 0 );
    DeleteDC(hdc);
    SetWindowPos(hdlg, HWND_TOP, pt->x, pt->y, 0,0, SWP_NOSIZE);
  } else
    CenterDialog(hdlg);
}

static void SaveWindowPosition( HWND hdlg, POINT *pt )
{
  RECT saverect;
  GetWindowRect(hdlg, &saverect);
  pt->x = saverect.left;
  pt->y = saverect.top;
}

EXPORT int GraphicFilterInitialize( MINIGRAPHICPARAM *param )
{
  DWORD type, len;

  // First, check that we can grok this plugin version!
  if (!param) return FALSE;
  if (param->version != PLUGINVERSION) return FALSE;

  InitCommonControls();
  StringCchCopy(param->title, MAXTITLE, "Flip and Rotate");
  param->version = PLUGINVERSION;
  param->user = NULL;

  type = REG_BINARY;
  len = sizeof(POINT);
  RegQueryValueEx(param->userKey, "Position", 0, &type,
		  (unsigned char *)&position, &len);
	
  return TRUE;
}

static BOOL CALLBACK Dialog(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static MINIGRAPHICPARAM *param = NULL;
  static MINIGRAPHIC *mg = NULL;
  static RECT rect = { 0, 0, 0, 0 };
  static RECT disp = { 0, 0, 0, 0 };

  int fliph, flipv, rot;

  switch(msg) {

  case WM_USER:
    fliph = SendDlgItemMessage(hDlg, IDC_H, BM_GETCHECK, 0 , 0)==BST_CHECKED?
      TRUE:FALSE;
    flipv = SendDlgItemMessage(hDlg, IDC_V, BM_GETCHECK, 0 , 0)==BST_CHECKED?
      TRUE:FALSE;

    if (SendDlgItemMessage(hDlg, IDC_0, BM_GETCHECK, 0, 0)==BST_CHECKED)
      rot = 0;
    else if (SendDlgItemMessage(hDlg, IDC_90, BM_GETCHECK, 0, 0)==BST_CHECKED)
      rot = 90;
    else if (SendDlgItemMessage(hDlg, IDC_180, BM_GETCHECK, 0, 0)==BST_CHECKED)
      rot = 180;
    else if (SendDlgItemMessage(hDlg, IDC_270, BM_GETCHECK, 0, 0)==BST_CHECKED)
      rot = 270;
    else
      rot = 0;
    if (mg) { param->FreeMiniGraphic(mg); mg = NULL; }
    mg = ImageIt( param, FALSE, fliph, flipv, rot, gfx );
    if (rot==90 || rot==270) {
      disp.left = rect.left;
      disp.right = disp.left + abs(rect.bottom-rect.top);
      disp.top = rect.top;
      disp.bottom = disp.top - abs(rect.right-rect.left);
    } else
      disp = rect;
    param->SetDisplayRect(&disp);
    param->PreviewMiniGraphic( mg );
    return TRUE;

  case WM_INITDIALOG:
    param = (MINIGRAPHICPARAM*)lParam;
    PositionWindow(hDlg, &position);
    mg = NULL;
		
    param->GetDisplayRect( &rect );
    SendDlgItemMessage(hDlg, IDC_H, BM_SETCHECK, BST_UNCHECKED, 0);
    SendDlgItemMessage(hDlg, IDC_V, BM_SETCHECK, BST_UNCHECKED, 0);

    SendDlgItemMessage(hDlg, IDC_0, BM_SETCHECK, BST_CHECKED, 0);
    SendDlgItemMessage(hDlg, IDC_90, BM_SETCHECK, BST_UNCHECKED, 0);
    SendDlgItemMessage(hDlg, IDC_180, BM_SETCHECK, BST_UNCHECKED, 0);
    SendDlgItemMessage(hDlg, IDC_270, BM_SETCHECK, BST_UNCHECKED, 0);

    SetFocus(GetDlgItem(hDlg, IDC_H));
    SendMessage(hDlg, WM_USER, 0, 0);
    return FALSE;


  case WM_COMMAND:
    switch (LOWORD(wParam)) {
    case IDOK:
      SaveWindowPosition(hDlg, &position);
      param->FreeMiniGraphic( gfx );
      gfx = mg;
      param->SetDisplayRect(&disp);
      EndDialog(hDlg, TRUE);
      return TRUE;
		
    case IDCANCEL:
      SaveWindowPosition(hDlg, &position);
      param->FreeMiniGraphic( mg );
      param->SetDisplayRect(&rect);
      EndDialog(hDlg, FALSE);
      return TRUE;

    case IDC_H:
    case IDC_V:
    case IDC_0:
    case IDC_90:
    case IDC_180:
    case IDC_270:
      PostMessage(hDlg, WM_USER, 0, 0);
      return TRUE;
    }
    break;
  }
  return FALSE;
}

EXPORT MINIGRAPHIC *GraphicFilterUserInterface( MINIGRAPHICPARAM *param,
						HWND parent,
						MINIGRAPHIC *graphic )
{
  gfx = graphic;
  DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG), parent,
		 Dialog, (LPARAM)param);
  return gfx;
}



EXPORT void GraphicFilterUninitialize( MINIGRAPHICPARAM *param )
{
  RegSetValueEx(param->userKey, "Position", 0, REG_BINARY,
		(unsigned char*)&position, sizeof(POINT));
}

